"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator.throw(value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments)).next());
    });
};
const assert = require('power-assert');
const sinon = require('sinon');
const mongodb_1 = require('mongodb');
const host = process.env.MONGODB_HOST || '127.0.0.1';
const port = process.env.MONGODB_PORT || 27017;
const mongo_entity_1 = require('../mongo_entity');
const collectionName = 'Collection';
describe('entity', () => {
    let db = null;
    const connector = {
        collection(name) { return db.collection(name); }
    };
    before(function () {
        return __awaiter(this, void 0, void 0, function* () {
            const name = "tmp" + Math.floor(Math.random() * 10000);
            db = yield mongodb_1.MongoClient.connect(`mongodb://${host}:${port}/${name}`);
            db['dispose'] = function () {
                return __awaiter(this, void 0, void 0, function* () {
                    yield db.dropDatabase();
                });
            };
        });
    });
    after(function () {
        db['dispose']();
        db.close();
    });
    it('contains connector and collection name', () => {
        const entity = new mongo_entity_1.default(connector, collectionName);
        assert.equal(entity._collectionName, collectionName);
        assert.equal(entity.connector, connector);
        assert.notEqual(entity.collection, null);
    });
    it('can find a multiple record', () => {
        const entity = new mongo_entity_1.default(connector, collectionName);
        const find = sinon.spy(entity.collection, 'find');
        const selector = {};
        const fields = {};
        const skip = 1;
        const limit = 2;
        const timeout = 1000;
        entity.find(selector, fields, skip, limit, timeout);
        assert(find.calledWithExactly(selector, fields, skip, limit, timeout));
    });
    it('can find a sinlge record', () => {
        const entity = new mongo_entity_1.default(connector, collectionName);
        const find = sinon.spy(entity.collection, 'find');
        const selector = {};
        const options = {};
        entity.findOne(selector, options);
        assert(find.calledWithExactly(selector, options));
    });
    it('can find and cache results of finding a single item', () => __awaiter(this, void 0, void 0, function* () {
        const entity = new mongo_entity_1.default(connector, collectionName);
        entity.collection.insertOne({ _id: '1' });
        const find = sinon.spy(entity.collection, 'findOne');
        let result = yield entity.findOne({ _id: '1' });
        result = yield entity.findOne({ _id: '1' });
        assert.deepEqual(result, { _id: '1' });
        assert(find.calledTwice);
        find.reset();
        result = yield entity.findOneCachedById('1');
        result = yield entity.findOneCachedById('1');
        assert(find.calledOnce);
    }));
    it('can find and cache results of finding multiple items', () => __awaiter(this, void 0, void 0, function* () {
        const entity = new mongo_entity_1.default(connector, collectionName);
        entity.collection.insertOne({ _id: '1' });
        const find = sinon.spy(entity.collection, 'find');
        let result = yield entity.findManyCached();
        result = yield entity.findManyCached();
        assert.deepEqual(result, [{ _id: '1' }]);
        assert(find.calledOnce);
    }));
    it('clears the cache when a new document is inserted', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const entity = new mongo_entity_1.default(connector, collectionName);
            const find = sinon.spy(entity.collection, 'find');
            entity.insert({ _id: '2' });
            let result = yield entity.findManyCached();
            entity.insert({ _id: '2' });
            result = yield entity.findManyCached();
            assert(find.calledTwice);
            assert.deepEqual(result, [{ _id: '1' }, { _id: '2' }]);
        });
    });
    it('clears the related cache when a document is updated', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const entity = new mongo_entity_1.default(connector, collectionName);
            const findSpy = sinon.spy(entity.collection, 'findOne');
            const cacheSpy = sinon.spy(entity, 'clearUpdateCaches');
            entity.insert({ _id: '1', file: 'foo' });
            entity.insert({ _id: '2', file: 'bar' });
            let foo = yield entity.findOneCachedById('1');
            let bar = yield entity.findOneCachedById('2');
            const selector = { _id: '1' };
            entity.update(selector, { $set: { file: 'boo' } });
            assert(cacheSpy.calledWith(selector));
            findSpy.reset();
            foo = yield entity.findOneCachedById('1');
            assert.deepEqual(foo, { _id: '1', file: 'boo' });
            assert(findSpy.calledOnce);
            findSpy.reset();
            bar = yield entity.findOneCachedById('2');
            assert(findSpy.notCalled);
        });
    });
    it('clears all caches when a document is updated and selector is unknown', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const entity = new mongo_entity_1.default(connector, collectionName);
            entity.insert({ _id: '1', file: 'foo' });
            entity.insert({ _id: '2', file: 'bar' });
            let foo = yield entity.findOneCachedById('1');
            let bar = yield entity.findOneCachedById('2');
            entity.update({ file: 'foo' }, { $set: { file: 'boo' } });
            const findSpy = sinon.spy(entity.collection, 'findOne');
            foo = yield entity.findOneCachedById('1');
            bar = yield entity.findOneCachedById('2');
            assert(findSpy.calledTwice);
        });
    });
});
